@@ -4,14 +4,16 @@
 #include <Animal.h>
 #include <string>
 
-// Note: The implementation is in the header for simplicity only...
-
 class Dog : public Animal {
 public:
   Dog(const std::string& name);
 
   void Bark() const;
   void MakeSound() const;
+
+  Dog* Clone() const;
+
+  static void MakeDogBark(const Dog& d);
 };
 
 
